package com.link.core.converter;

import com.link.core.converter.i18n.DefaultStringValue;
import com.link.core.converter.i18n.Resources;
import com.link.core.converter.util.RegularCloneable;
import com.link.core.converter.util.StringUtil;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtField;
import javassist.CtMethod;
import javassist.NotFoundException;

public class ConverterBuilder<S, T> {
   private static final String BUILDER_FIELD_NAME = "builder_";
   private static final String BUILDER_GETTER_METHOD_NAME = "getBuilder";
   private static final String BUILDER_SETTER_METHOD_NAME = "setBuilder";
   private static final String INJECTOR_FIELD_NAME = "injector_";
   private static final String INJECTOR_GETTER_METHOD_NAME = "getInjector";
   private static final String INJECTOR_SETTER_METHOD_NAME = "setInjector";
   private static final String GET_CONVERTERS_METHOD_NAME = "getConverters";
   private static final String INJECT_METHOD_NAME = "inject";
   private static final String CONVERT_METHOD_NAME = "convert";
   private static final String COPY_FROM_METHOD_NAME = "copyFrom";
   private static final String CLONE_METHOD_NAME = "clone";
   private InjectorBuilder<S, T> injectorBuilder;

   public static <S, T> ConverterBuilder<S, T> newBuilder(Class<S> sourceType, Class<T> targetType) throws IllegalArgumentException {
      return new ConverterBuilder(sourceType, targetType);
   }

   static CtClass getCtClass(ClassPool cp, Class type) throws IllegalArgumentException, NotFoundException {
      return InjectorBuilder.getCtClass(cp, type);
   }

   static CtClass getCtClass(ClassPool cp, String className) throws IllegalArgumentException {
      return InjectorBuilder.getCtClass(cp, className);
   }

   private ConverterBuilder() {
   }

   ConverterBuilder(Class<? extends S> sourceType, Class<? extends T> targetType) throws IllegalArgumentException {
      this.injectorBuilder = new InjectorBuilder(sourceType, targetType);
   }

   public Converter<S, T> build() throws ConversionException {
      return (Converter)this.build((Class)null);
   }

   public <C> C build(Class<?> superType) throws IllegalArgumentException, ConversionException {
      try {
         Converter<S, T> converter = (Converter)this.buildType(superType).newInstance();
         ((HasConverterBuilder)converter).setBuilder(this);
         ((HasInjector)converter).setInjector(this.injectorBuilder.build());
         return converter;
      } catch (ConversionException var3) {
         ConversionException e = var3;
         throw e;
      } catch (Exception var4) {
         Exception e = var4;
         throw new ConversionException(e);
      }
   }

   private Class<? extends Converter<S, T>> buildType(Class<?> superType) throws ConversionException {
      try {
         Class<?> ripeSuperType = superType == null ? Object.class : superType;
         if (ripeSuperType.isInterface()) {
            throw new IllegalArgumentException(MessageFormat.format(ConverterBuilder.R.R.cannotBeInterface(), "superType"));
         } else {
            ClassPool cp = ClassPool.getDefault();
            String className = this.generateClassName(superType);
            CtClass converterClass = getCtClass(cp, className);
            if (converterClass != null) {
               return Class.forName(className);
            } else {
               converterClass = cp.makeClass(className);
               CtClass superClass = getCtClass(cp, ripeSuperType);
               converterClass.setSuperclass(superClass);
               this.buildConstructor(cp, converterClass);
               this.realizeHasConverterBuilder(cp, converterClass, ripeSuperType);
               this.realizeHasInjector(cp, converterClass, ripeSuperType);
               this.realizeHasConverters(cp, converterClass, ripeSuperType);
               this.realizeInjector(cp, converterClass, ripeSuperType);
               this.realizeConverter(cp, converterClass, ripeSuperType);
               this.realizeRegularCloneable(cp, converterClass, ripeSuperType);
               this.buildCloneMethod(cp, converterClass);
               return converterClass.toClass();
            }
         }
      } catch (ConversionException var7) {
         ConversionException e = var7;
         throw e;
      } catch (Exception var8) {
         Exception e = var8;
         throw new ConversionException(e);
      }
   }

   public ConverterBuilder<S, T> map(String sourcePropertyName, String targetPropertyName, Converter<?, ?> converter) {
      this.injectorBuilder.map(sourcePropertyName, targetPropertyName, converter);
      return this;
   }

   public ConverterBuilder<S, T> map(String sourcePropertyName, String targetPropertyName) {
      this.injectorBuilder.map(sourcePropertyName, targetPropertyName);
      return this;
   }

   public ConverterBuilder<S, T> map(String propertyName, Converter<?, ?> converter) {
      this.injectorBuilder.map(propertyName, converter);
      return this;
   }

   public <SP, TP> ConverterBuilder<S, T> map(Class<SP> sourcePropertyType, Class<TP> targetPropertyType, Converter<?, ?> converter) throws IllegalArgumentException {
      this.injectorBuilder.map(sourcePropertyType, targetPropertyType, converter);
      return this;
   }

   public <SP, TP> ConverterBuilder<S, T> map(Class<SP> sourcePropertyType, Class<TP> targetPropertyType, Converter<?, ?> converter, boolean polymorphic) throws IllegalArgumentException {
      this.injectorBuilder.map(sourcePropertyType, targetPropertyType, converter, polymorphic);
      return this;
   }

   public <SP, TP> ConverterBuilder<S, T> map(Class<SP> sourcePropertyType, boolean sourcePolymorphic, Class<TP> targetPropertyType, boolean targetPolymorphic, Converter<?, ?> converter) throws IllegalArgumentException {
      this.injectorBuilder.map(sourcePropertyType, sourcePolymorphic, targetPropertyType, targetPolymorphic, converter);
      return this;
   }

   public ConverterBuilder<S, T> ignore(String... targetPropertyNames) {
      this.injectorBuilder.ignore(targetPropertyNames);
      return this;
   }

   public ConverterBuilder<S, T> inclouds(String... targetPropertyNames) {
      this.injectorBuilder.inclouds(targetPropertyNames);
      return this;
   }

   public ConverterBuilder<S, T> extend(Injector<?, ?> injector) throws IllegalArgumentException {
      this.injectorBuilder.extend(injector);
      return this;
   }

   public ConverterBuilder<S, T> addFilter(InjectorFilter<? extends S, ? extends T> filter) throws IllegalArgumentException {
      this.injectorBuilder.addFilter(filter);
      return this;
   }

   public ConverterBuilder<S, T> clone() {
      ConverterBuilder<S, T> other = new ConverterBuilder();
      other.injectorBuilder = this.injectorBuilder.clone();
      return other;
   }

   private String generateClassName(Class<?> superType) {
      StringBuffer sb = new StringBuffer();
      sb.append(this.injectorBuilder.getTargetType().getName());
      sb.append("Converter$");
      sb.append(this.injectorBuilder.getSourceType().getName().replace('.', '_'));
      sb.append('$');
      int prime = true;
      int feature = 1;
      feature = 31 * feature + this.injectorBuilder.getTypeFeature();
      feature = 31 * feature + (superType == null ? 0 : superType.getName().hashCode());
      sb.append(Integer.toHexString(feature));
      return sb.toString();
   }

   private void buildConstructor(ClassPool cp, CtClass converterClass) throws CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      CtConstructor constructor = new CtConstructor(new CtClass[0], converterClass);
      constructor.setModifiers(1);
      constructor.setBody("{}");
      converterClass.addConstructor(constructor);
   }

   private void realizeHasConverterBuilder(ClassPool cp, CtClass converterClass, Class<?> superType) throws NotFoundException, CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      assert superType != null;

      CtClass gettableClass = getCtClass(cp, HasConverterBuilder.class);
      if (!this.realizes(superType, HasConverterBuilder.class)) {
         converterClass.addInterface(gettableClass);
      }

      Method superGetterMethod = null;

      try {
         superGetterMethod = superType.getMethod("getBuilder");
         if (superGetterMethod != null && (!ConverterBuilder.class.equals(superGetterMethod.getReturnType()) || Modifier.isPrivate(superGetterMethod.getModifiers()))) {
            superGetterMethod = null;
         }
      } catch (NoSuchMethodException var12) {
      }

      Method superSetterMethod = null;

      try {
         superSetterMethod = superType.getMethod("setBuilder", ConverterBuilder.class);
         if (superSetterMethod != null && Modifier.isPrivate(superSetterMethod.getModifiers())) {
            superSetterMethod = null;
         }
      } catch (NoSuchMethodException var11) {
      }

      CtClass builderClass = getCtClass(cp, ConverterBuilder.class);
      CtField builderField = new CtField(builderClass, "builder_", converterClass);
      builderField.setModifiers(2);
      converterClass.addField(builderField);
      CtMethod getterMethod = new CtMethod(builderClass, "getBuilder", new CtClass[0], converterClass);
      getterMethod.setModifiers(1);
      getterMethod.setBody(this.buildGetBuilderBody(superGetterMethod != null));
      converterClass.addMethod(getterMethod);
      CtMethod setterMethod = new CtMethod(CtClass.voidType, "setBuilder", new CtClass[]{builderClass}, converterClass);
      setterMethod.setModifiers(1);
      setterMethod.setBody(this.buildSetBuilderBody(superSetterMethod != null));
      converterClass.addMethod(setterMethod);
   }

   private String buildSetBuilderBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      sb.append("  ");
      sb.append(ConverterBuilder.class.getName());
      sb.append(" builder = (");
      sb.append(ConverterBuilder.class.getName());
      sb.append(") $1;\n");
      if (callSuper) {
         sb.append("  super.");
         sb.append("setBuilder");
         sb.append("(builder);\n");
      }

      sb.append("  ");
      sb.append("builder_");
      sb.append(" = builder;\n");
      sb.append("}");
      return sb.toString();
   }

   private String buildGetBuilderBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      if (callSuper) {
         sb.append("  super.");
         sb.append("getBuilder");
         sb.append("();\n");
      }

      sb.append("  return ");
      sb.append("builder_");
      sb.append(";\n");
      sb.append("}");
      return sb.toString();
   }

   private void realizeHasInjector(ClassPool cp, CtClass converterClass, Class<?> superType) throws NotFoundException, CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      assert superType != null;

      CtClass hasInjectorClass = getCtClass(cp, HasInjector.class);
      if (!this.realizes(superType, HasInjector.class)) {
         converterClass.addInterface(hasInjectorClass);
      }

      Method superGetterMethod = null;

      try {
         superGetterMethod = superType.getMethod("getInjector");
         if (superGetterMethod != null && (!Injector.class.equals(superGetterMethod.getReturnType()) || Modifier.isPrivate(superGetterMethod.getModifiers()))) {
            superGetterMethod = null;
         }
      } catch (NoSuchMethodException var12) {
      }

      Method superSetterMethod = null;

      try {
         superSetterMethod = superType.getMethod("setInjector", Injector.class);
         if (superSetterMethod != null && Modifier.isPrivate(superSetterMethod.getModifiers())) {
            superSetterMethod = null;
         }
      } catch (NoSuchMethodException var11) {
      }

      CtClass injectorClass = getCtClass(cp, Injector.class);
      CtField injectorField = new CtField(injectorClass, "injector_", converterClass);
      injectorField.setModifiers(2);
      converterClass.addField(injectorField);
      CtMethod getterMethod = new CtMethod(injectorClass, "getInjector", new CtClass[0], converterClass);
      getterMethod.setModifiers(1);
      getterMethod.setBody(this.buildGetInjectorBody(superGetterMethod != null));
      converterClass.addMethod(getterMethod);
      CtMethod setterMethod = new CtMethod(CtClass.voidType, "setInjector", new CtClass[]{injectorClass}, converterClass);
      setterMethod.setModifiers(1);
      setterMethod.setBody(this.buildSetInjectorBody(superSetterMethod != null));
      converterClass.addMethod(setterMethod);
   }

   private String buildGetInjectorBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      if (callSuper) {
         sb.append("  super.");
         sb.append("getInjector");
         sb.append("();\n");
      }

      sb.append("  return ");
      sb.append("injector_");
      sb.append(";\n");
      sb.append("}");
      return sb.toString();
   }

   private String buildSetInjectorBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      sb.append("  ");
      sb.append(Injector.class.getName());
      sb.append(" injector = (");
      sb.append(Injector.class.getName());
      sb.append(") $1;\n");
      if (callSuper) {
         sb.append("  super.");
         sb.append("setInjector");
         sb.append("(injector);\n");
      }

      sb.append("  ");
      sb.append("injector_");
      sb.append(" = injector;\n");
      sb.append("}");
      return sb.toString();
   }

   private void realizeHasConverters(ClassPool cp, CtClass converterClass, Class<?> superType) throws NotFoundException, CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      assert superType != null;

      CtClass hasConvertersClass = getCtClass(cp, HasConverters.class);
      if (!this.realizes(superType, HasConverters.class)) {
         converterClass.addInterface(hasConvertersClass);
      }

      Method superMethod = null;

      try {
         superMethod = superType.getMethod("getConverters");
         if (superMethod != null && Modifier.isPrivate(superMethod.getModifiers())) {
            superMethod = null;
         }
      } catch (NoSuchMethodException var8) {
      }

      CtClass listClass = getCtClass(cp, List.class);
      CtMethod method = new CtMethod(listClass, "getConverters", new CtClass[0], converterClass);
      method.setModifiers(1);
      method.setBody(this.buildGetConvertersBody(superMethod != null));
      converterClass.addMethod(method);
   }

   private String buildGetConvertersBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      if (callSuper) {
         sb.append("  ");
         sb.append(List.class.getName());
         sb.append(" list = super.");
         sb.append("getConverters");
         sb.append("();\n");
      } else {
         sb.append("  ");
         sb.append(List.class.getName());
         sb.append(" list = new ");
         sb.append(ArrayList.class.getName());
         sb.append("();\n");
      }

      sb.append("  ");
      sb.append(Injector.class.getName());
      sb.append(" injector = ");
      sb.append("getInjector");
      sb.append("();\n");
      sb.append("  if (injector instanceof ");
      sb.append(HasConverters.class.getName());
      sb.append(") {\n");
      sb.append("    ");
      sb.append(List.class.getName());
      sb.append(" list2 = ((");
      sb.append(HasConverters.class.getName());
      sb.append(") injector).");
      sb.append("getConverters");
      sb.append("();\n");
      sb.append("    list.addAll(list2);\n");
      sb.append("  }\n");
      sb.append("  return list;\n");
      sb.append("}");
      return sb.toString();
   }

   private void realizeInjector(ClassPool cp, CtClass converterClass, Class<?> superType) throws NotFoundException, CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      assert superType != null;

      CtClass injectorClass = getCtClass(cp, Injector.class);
      boolean inheritedInterface = true;
      if (!this.realizes(superType, Injector.class)) {
         converterClass.addInterface(injectorClass);
         inheritedInterface = false;
      }

      Method superInjectMethod = null;

      try {
         superInjectMethod = superType.getMethod("inject", Object.class, Object.class);
         if (superInjectMethod != null && Modifier.isPrivate(superInjectMethod.getModifiers())) {
            superInjectMethod = null;
         }
      } catch (NoSuchMethodException var12) {
      }

      Class sourceType = inheritedInterface ? this.injectorBuilder.getSourceType() : Object.class;
      Class targetType = inheritedInterface ? this.injectorBuilder.getTargetType() : Object.class;
      CtClass sourceClass = getCtClass(cp, sourceType);
      CtClass targetClass = getCtClass(cp, targetType);
      CtMethod injectMethod = new CtMethod(CtClass.voidType, "inject", new CtClass[]{sourceClass, targetClass}, converterClass);
      injectMethod.setModifiers(1);
      injectMethod.setBody(this.buildInjectBody(superInjectMethod != null));
      converterClass.addMethod(injectMethod);
   }

   private String buildInjectBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      Class sourceType = this.injectorBuilder.getSourceType();
      sb.append("  ");
      sb.append(sourceType.getName());
      sb.append(" source = (");
      sb.append(sourceType.getName());
      sb.append(") $1;\n");
      Class targetType = this.injectorBuilder.getTargetType();
      sb.append("  ");
      sb.append(targetType.getName());
      sb.append(" target = (");
      sb.append(targetType.getName());
      sb.append(") $2;\n");
      if (callSuper) {
         sb.append("  super.inject(source, target);\n");
      } else {
         sb.append("  if (source == null) {\n");
         sb.append("    throw new IllegalArgumentException(\"The parameter 'source' cannot be null.\");\n");
         sb.append("  }\n");
         sb.append("  if (target == null) {\n");
         sb.append("    throw new IllegalArgumentException(\"The parameter 'target' cannot be null.\");\n");
         sb.append("  }\n");
      }

      sb.append("  ((");
      sb.append(Injector.class.getName());
      sb.append(") ");
      sb.append("getInjector");
      sb.append("()).inject(source, target);\n");
      sb.append("}");
      return sb.toString();
   }

   private void realizeConverter(ClassPool cp, CtClass converterClass, Class<?> superType) throws NotFoundException, CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      assert superType != null;

      CtClass interfaceClass = getCtClass(cp, Converter.class);
      boolean inheritedInterface = true;
      if (!this.realizes(superType, Converter.class)) {
         converterClass.addInterface(interfaceClass);
         inheritedInterface = false;
      }

      Class sourceType = inheritedInterface ? this.injectorBuilder.getSourceType() : Object.class;
      Method superConvertMethod = null;

      try {
         superConvertMethod = superType.getMethod("convert", sourceType);
         if (superConvertMethod != null && Modifier.isPrivate(superConvertMethod.getModifiers())) {
            superConvertMethod = null;
         }
      } catch (NoSuchMethodException var10) {
      }

      CtClass objectClass = getCtClass(cp, Object.class);
      CtMethod convertMethod = new CtMethod(objectClass, "convert", new CtClass[]{objectClass}, converterClass);
      convertMethod.setModifiers(1);
      convertMethod.setBody(this.buildConvertBody(superConvertMethod != null));
      converterClass.addMethod(convertMethod);
   }

   private String buildConvertBody(boolean callSuper) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      Class sourceType = this.injectorBuilder.getSourceType();
      sb.append("  ");
      sb.append(sourceType.getName());
      sb.append(" source = (");
      sb.append(sourceType.getName());
      sb.append(") $1;\n");
      sb.append("  if (source == null) {\n");
      sb.append("    return null;\n");
      sb.append("  }\n");
      Class targetType = this.injectorBuilder.getTargetType();
      if (callSuper) {
         sb.append("  ");
         sb.append(targetType.getName());
         sb.append(" target = (");
         sb.append(targetType.getName());
         sb.append(") super.convert(source);\n");
         sb.append("  if (target == null) {\n");
         sb.append("    return null;\n");
         sb.append("  }\n");
      } else {
         sb.append("  ");
         sb.append(targetType.getName());
         sb.append(" target = new ");
         sb.append(targetType.getName());
         sb.append("();\n");
      }

      sb.append("  inject(source, target);\n");
      sb.append("  return target;\n");
      sb.append("}");
      return sb.toString();
   }

   private void realizeRegularCloneable(ClassPool cp, CtClass converterClass, Class<?> superType) throws NotFoundException, CannotCompileException {
      assert cp != null;

      assert converterClass != null;

      assert superType != null;

      CtClass interfaceClass = getCtClass(cp, RegularCloneable.class);
      if (!this.realizes(superType, RegularCloneable.class)) {
         converterClass.addInterface(interfaceClass);
      }

      Method superCopyFromMethod = null;

      try {
         superCopyFromMethod = superType.getMethod("copyFrom", Object.class);
         if (superCopyFromMethod != null && Modifier.isPrivate(superCopyFromMethod.getModifiers())) {
            superCopyFromMethod = null;
         }
      } catch (NoSuchMethodException var9) {
      }

      CtClass objectClass = getCtClass(cp, Object.class);
      CtMethod copyFromMethod = new CtMethod(CtClass.voidType, "copyFrom", new CtClass[]{objectClass}, converterClass);
      copyFromMethod.setModifiers(1);
      boolean callSuper = superCopyFromMethod != null;
      copyFromMethod.setBody(this.buildCopyFromBody(callSuper, converterClass.getName()));
      converterClass.addMethod(copyFromMethod);
   }

   private String buildCopyFromBody(boolean callSuper, String converterClassName) {
      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      if (callSuper) {
         sb.append("  super.");
         sb.append("copyFrom");
         sb.append("($1);\n");
      }

      sb.append("  if ($1 == null || $1 instanceof ");
      sb.append(converterClassName);
      sb.append(" == false) {\n");
      sb.append("    return;\n");
      sb.append("  }\n");
      sb.append("  ");
      sb.append(converterClassName);
      sb.append(" source = (");
      sb.append(converterClassName);
      sb.append(") $1;\n");
      sb.append("  ");
      sb.append(Injector.class.getName());
      sb.append(" injector = ((");
      sb.append(InjectorBuilder.InjectorAssist.class.getName());
      sb.append(") source.");
      sb.append("getInjector");
      sb.append("()).clone();\n");
      sb.append("  ");
      sb.append("setInjector");
      sb.append("(injector);\n");
      sb.append("  ");
      sb.append(ConverterBuilder.class.getName());
      sb.append(" builder = (");
      sb.append(ConverterBuilder.class.getName());
      sb.append(") ((");
      sb.append(HasConverterBuilder.class.getName());
      sb.append(") source).");
      sb.append("getBuilder");
      sb.append("();\n");
      sb.append("  ");
      sb.append("setBuilder");
      sb.append("(builder);\n");
      sb.append("}");
      return sb.toString();
   }

   private void buildCloneMethod(ClassPool cp, CtClass converterClass) throws CannotCompileException, NotFoundException {
      assert cp != null;

      assert converterClass != null;

      CtClass objectClass = getCtClass(cp, Object.class);
      CtMethod cloneMethod = new CtMethod(objectClass, "clone", new CtClass[0], converterClass);
      cloneMethod.setModifiers(1);
      cloneMethod.setBody(this.buildCloneBody(converterClass.getName()));
      converterClass.addMethod(cloneMethod);
   }

   private String buildCloneBody(String converterClassName) {
      assert !StringUtil.isNullOrBlank(converterClassName);

      StringBuffer sb = new StringBuffer();
      sb.append("{\n");
      sb.append("  ");
      sb.append(converterClassName);
      sb.append(" other = new ");
      sb.append(converterClassName);
      sb.append("();\n");
      sb.append("  ((");
      sb.append(RegularCloneable.class.getName());
      sb.append(") other).");
      sb.append("copyFrom");
      sb.append("($0);\n");
      sb.append("  return other;\n");
      sb.append("}");
      return sb.toString();
   }

   private boolean realizes(Class<?> type, Class<?> interfaceType) {
      assert type != null;

      assert interfaceType != null;

      Class[] var3 = type.getInterfaces();
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         Class<?> intfType = var3[var5];
         if (interfaceType.equals(intfType)) {
            return true;
         }
      }

      return false;
   }

   public interface R {
      R R = (R)Resources.create(R.class);

      @DefaultStringValue("参数{0}禁止为接口。")
      String cannotBeInterface();
   }
}
